//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file NeighborCache.h
 * @author Antonio Zea
 */


#ifndef __NEIGHBORCACHE_H_
#define __NEIGHBORCACHE_H_

#include <omnetpp.h>

#include <map>

#include <NodeHandle.h>

class TransportAddress;
class RpcListener;

enum {
    NEIGHBORSTATE_UNKNOWN = -1,
    NEIGHBORSTATE_TIMEOUT = -2,
    NEIGHBORSTATE_WAITING = -3
};

struct WaitingContexts {
//    ~WaitingContexts()
//    {
//        for (uint i = 0; i < pingContexts.size(); ++i) {
//            delete pingContexts[i];
//        }
//    };

    std::vector<RpcListener*> pingListeners;
    std::vector<cPolymorphic*> pingContexts;
    std::vector<uint> pingIds;
};

class NeighborCache : public cSimpleModule
{
private:
    bool enableNeighborCache;
    simtime_t rttExpirationTime;
    uint maxSize;

    bool cleanupCache();

    void updateEntry(const TransportAddress& address,
                     simtime_t insertTime);

protected:


    struct NeighborCacheEntry {
        NeighborCacheEntry()
        {
            waitingContexts = NULL;
        };

//        ~NeighborCacheEntry()
//        {
//            //delete waitingContexts;
//        };

        simtime_t  insertTime;
        simtime_t  rtt;
        NodeHandle nodeRef;
        WaitingContexts* waitingContexts;
        std::vector<double> coordinates;
    };

    std::map<TransportAddress, NeighborCacheEntry> neighborCache;
    std::multimap<simtime_t, TransportAddress> neighborCacheExpireMap;

    void initialize();

public:
    ~NeighborCache();

    inline bool isEnabled()
    {
        return enableNeighborCache;
    };

    bool insertPingedNode(const TransportAddress& handle,
                          cPolymorphic* context,
                          RpcListener* rpcListener,
                          int rpcId);

    inline uint numInhibitedPings(const TransportAddress& handle)
    {
        if (neighborCache.count(handle) == 0 || !par("enableNeighborCache") ||
            neighborCache[handle].waitingContexts == NULL)
            return 0;
        return neighborCache[handle].waitingContexts->pingIds.size();
    };

    inline WaitingContexts* getWaitingContexts(const TransportAddress& handle)
    {
        if (!enableNeighborCache) return NULL;
        if (neighborCache.count(handle) == 0)
            throw new cRuntimeError("NeighborCache error!");
        WaitingContexts* temp = neighborCache[handle].waitingContexts;
        neighborCache[handle].waitingContexts = NULL;
        return temp;
    };

    void insertAddressRtt(const NodeHandle &add, simtime_t time);
    void addressTimeout(const TransportAddress& handle);
    simtime_t getAddressRtt(const TransportAddress &add);
    NodeHandle getNodeHandle(const TransportAddress &add);
    //void clearNodeContexts(const TransportAddress& handle);

    friend std::ostream& operator<<(std::ostream& os, const NeighborCacheEntry &);
};

#endif
