//
// Copyright (C) 2008 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#include <omnetpp.h>
#include <cnetcommbuffer.h>
#include "OverSimMessage_m.h"
#include "OverSimMessage.h"

Register_Class(OverSimMessage);

void OverSimMessage::netPack(cCommBuffer *buffer)
{
    cObject::netPack(buffer);

    if (contextPointer() || controlInfo())
        throw new cRuntimeError(this, "netPack(): cannot pack object with "
                                "contextPointer or controlInfo set");
    if (parList().items() > 0) {
        buffer->packFlag(true);
        buffer->packObject(&parList());
    } else {
        buffer->packFlag(false);
    }
    
    if (buffer->packFlag(encapsulatedMsg() != NULL))
        buffer->packObject(encapsulatedMsg());
}

void OverSimMessage::netUnpack(cCommBuffer *buffer)
{
    int len = 0;
    
    cNetCommBuffer *netBuffer = dynamic_cast<cNetCommBuffer*>(buffer);
    if (netBuffer != NULL) {
        len = netBuffer->getRemainingMessageSize();
    }
    
    cObject::netUnpack(buffer);

    ASSERT(shareCount() == 0);

    if (buffer->checkFlag()) {
        cArray *parlistptr = static_cast<cArray*>(buffer->unpackObject());
        std::cout << "still there: " << *parlistptr << std::endl;
        for (int i=0; i<parlistptr->items(); i++) {
            std::cout << "i: " << i << " " << parlistptr->get(i) << std::endl;
            addObject(static_cast<cObject*>(parlistptr->get(i)->dup()));
        }
        delete parlistptr;
    }

    if (buffer->checkFlag()) {
        encapsulate((cMessage *) buffer->unpackObject());
    }
    
    // set the length of the received message
    // TODO: doesn't contain the length of the string for the object type
    setByteLength(len);
}
