//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file bamboo.h
 * @author Gerhard Petruschat
 */

#ifndef __BAMBOO_H_
#define __BAMBOO_H_

#include <vector>
#include <map>
#include <queue>
#include <algorithm>

#include <omnetpp.h>
#include <IPvXAddress.h>

#include <OverlayKey.h>
#include <NodeHandle.h>
#include <BaseOverlay.h>
#include <BasePastry.h>

#include "PastryTypes.h"
#include "PastryMessage_m.h"
#include "PastryRoutingTable.h"
#include "PastryLeafSet.h"
#include "PastryNeighborhoodSet.h"



/**
 * Pastry overlay module
 *
 * @author Felix Palmen
 * @see BaseOverlay
 */
class Bamboo : public BasePastry
{
  public:
    virtual ~Bamboo();

    // see BaseOverlay.h
    virtual void initializeOverlay(int stage);

    // see BaseOverlay.h
    virtual void handleTimerEvent(cMessage* msg);

    // see BaseOverlay.h
    virtual void handleUDPMessage(BaseOverlayMessage* msg);

    void handleStateMessage(PastryStateMessage* msg);

  protected:

    /**
     * changes node state
     *
     * @param toState state to change to
     */
    virtual void changeState(int toState);

    virtual void lookupFinished(AbstractLookup *lookup);

    friend class BambooLookupListener;

  private:

       // local state tables
    double leafsetMaintenanceTimeoutAmount;
    double globalTuningTimeoutAmount;
   // uint rowToAsk;

    cMessage* leafsetMaintenanceTimeout;
    cMessage* globalTuningTimeout;

    /**
     * periodically repairs the leafset by pushing it to and pulling it from 
     * from a random live leafset node 
     */
    void doLeafsetMaintenance(void);

    /**
     * periodically repairs the routing table by performing random lookups
     */
    void doGlobalTuning(void);

    /**
     * notifies leafset and routingtable of a failed node and sends out
     * a repair request if possible
     *
     * @param failed the failed node
     * @return true as long as local state is READY (signals lookup to try
     * again)
     */
    bool handleFailedNode(const TransportAddress& failed);

     /**
     * checks whether proxCache is complete, takes appropriate actions
     * depending on the protocol state
     */
    void checkProxCache(void);

    // see BaseOverlay.h
    virtual void joinOverlay();
 
};


#endif
