//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file NeighborsList.h
 * @author Helge Backhaus
 */

#ifndef __NEIGHBORSLIST_H_
#define __NEIGHBORSLIST_H_

#include <map>
#include <set>
#include <omnetpp.h>
#include <NodeHandle.h>
#include <InitStages.h>

#include "VastDefs.h"
#include "Geometry.h"
#include "Edgelist.h"
#include "Heap.h"

typedef std::map<NodeHandle, Site> SiteMap;
typedef std::set<Vector2D> PositionSet;

/// NeighborsList class
/**
 Maintains a list of all neighbors and functions for building their voronoi diagram.
 */

class NeighborsList : public cSimpleModule
{
    public:
        virtual ~NeighborsList();
        virtual int numInitStages() const { return MAX_STAGE_OVERLAY + 1; }
        virtual void initialize(int stage);
        virtual void handleMessage(cMessage* msg);

        void initializeList(Vector2D p, NodeHandle owner, double AOI_size);
        void setDebug(bool debugOutput);
        void addNode(Vector2D p, NodeHandle node);
        void updateTimeStamp(NodeHandle node);
        void removeNode(NodeHandle node);
        void changePosition(Vector2D p);
        bool validatePosition(Vector2D p);
        void buildVoronoi();
        void buildVoronoi(Vector2D old_pos, Vector2D new_pos);
        void removeNeighbors();
        int getSize();
        void clearList();

        SiteMap Sites;
        Site thisSite;
    protected:
        PositionSet Positions;

        double AOI_size;
        bool debugOutput;

        // Voronoi parameters
        Geometry geom;
        EdgeList edgelist;
        HeapPQ heap;

        void updateDisplayString();
};

#endif
