//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file Vast.h
 * @author Helge Backhaus
 */

#ifndef __VAST_H_
#define __VAST_H_

#include <omnetpp.h>
#include <IPvXAddress.h>
#include <NodeHandle.h>
#include <BaseOverlay.h>
#include <float.h>

#include <Vast_m.h>
#include <NeighborsList.h>
#include <VastDefs.h>

/// Voronoi class
/**
 * An overlay network based on voronoi diagrams.
 */

class Vast : public BaseOverlay
{
    public:
        // OMNeT++
        virtual ~Vast();
        virtual void initializeOverlay(int stage);
        virtual void finishOverlay();
        virtual void handleUDPMessage(BaseOverlayMessage* msg);
        virtual void handleTimerEvent(cMessage* msg);
        virtual void handleAppMessage(cMessage* msg);
        virtual void receiveChangeNotification(int category, cPolymorphic *details);

        double getAOI();
        Vector2D getPosition();
        NeighborsList* getList();

    protected:
        // node references
        double AOI_size;
        Vector2D position;

        // module references
        NeighborsList* neighborsList;

        // statistics
        long joinRequestBytesSent,
             joinAcknowledgeBytesSent,
             nodeMoveBytesSent,
             newNeighborsBytesSent,
             nodeLeaveBytesSent,
             enclosingNeighborsRequestBytesSent,
             pingBytesSent,
             pongBytesSent,
             discardNodeBytesSent;

        long maxBytesPerSecondSent, averageBytesPerSecondSent, bytesPerSecond;

        // parameters
        bool debugVoronoiOutput, ignoreFalseKeys;
        double joinTimeout, pingTimeout;

        // timers
        cMessage* join_timer;
        cMessage* ping_timer;
        cMessage* sec_timer;

        void sendToApp(cMessage* msg);
        void sendMessage(VastMessage* vastMsg, NodeHandle destAddr);
        void setBootstrapedIcon();
        void changeState(int state);

        double distance_square(Site s, VastMessage *msg);

        // timer processing
        void processJoinTimer();
        void processPingTimer();
        void processSecTimer();

        // app handlers
        void handleJoin(GameAPIPositionMessage* sgcMsg);
        void handleMove(GameAPIPositionMessage* sgcMsg);
        void handleEvent( GameAPIMessage* msg );

        // overlay handlers
        void handleJoinRequest(VastMessage *vastMsg);
        void handleJoinAcknowledge(VastListMessage *vastListMsg);
        void handleNodeMove(VastMoveMessage *vastMoveMsg);
        void handleNewNeighbors(VastListMessage *vastListMsg);
        void handleNodeLeave(VastListMessage *vastListMsg);
        void handleEnclosingNeighborsRequest(VastMessage *vastMsg);
        void handlePing(VastMessage *vastMsg);
        void handlePong(VastMessage *vastMsg);
        void handleDiscardNode(VastDiscardMessage *vastMsg);
        void sendDiscardNode(VastMessage *vastMsg);
        void synchronizeApp(VastMoveMessage *vastMoveMsg = NULL);
};

#endif
