//
// Copyright (C) 2006 Institut fuer Telematik, Universitaet Karlsruhe (TH)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

/**
 * @file VastDefs.h
 * @author Helge Backhaus
 */

#ifndef __DEFS_H_
#define __DEFS_H_

#include <NodeHandle.h>
#include <Vector2D.h>

// constants for node types
#define UNDEF       0
#define THIS        1
#define ENCLOSING   2
#define NEIGHBOR    4
#define BOUNDARY    8
#define NEW        16

/// Vast Definitions
/**
 Some structures needed for building a voronoi diagram and maintaining the overlays neighbors.
 */

struct Site
{
    Site();
    Vector2D        coord;
    unsigned char   type;
    bool            innerEdge[3], outerEdge, isAdded;
    NodeHandle      addr;
    double          tstamp;
    friend std::ostream& operator<<(std::ostream& Stream, const Site s);
};

struct Edge
{
    Edge();
    double  a, b, c;
    Site    *ep[2];
    Site    *reg[2];
};

struct Halfedge
{
    Halfedge();
    Halfedge    *ELleft, *ELright;
    Edge        *ELedge;
    char        ELpm;
    Site        *vertex;
    double      ystar;
    Halfedge    *PQnext;
};

#endif
